﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Db4objects.Db4o;
using Db4objects.Db4o.Collections;
using Db4objects.Db4o.Linq;
using Db4objects.Db4o.TA;

namespace Db4OSeries
{
    public class Program
    {
        private const string FilePath = "database.db4o";

        internal static void Main(string[] args)
        {
            File.Delete(FilePath);
            using (var db = OpenDBWithTA(FilePath))
            {
                var pers = new Person
                               {
                                   House = new House {Name = "My House"},
                                   Gender = Gender.Female,
                                   MyPet = BuildDeepGraph()
                               };

                pers.Add(new Address("Main.Street", "A City"));

                // initial store
                db.Store(pers);
            }
            using (var db = OpenDBWithTA(FilePath))
            {
                var me = LoadPerson(db);

                var canLoadDeepObjectGraph = me.MyPet.Father.Father.Father.Father.Father;
                AssertTrue(null != canLoadDeepObjectGraph);

                // some changes, note, NO STORE
                me.Add(new Address("Holiday-House", "Other City"));
                me.Gender = Gender.Male;
                me.House.Name = "BiggerHouse";
            }
            using (var db = OpenDBWithTA(FilePath))
            {
                var me = LoadPerson(db);
                // everything is stored =)
                AssertTrue(me.Addresses.Count() == 2);
                AssertTrue(me.Gender == Gender.Male);
                AssertTrue(me.House.Name == "BiggerHouse");
            }

            Console.Out.WriteLine("Finished");
            Console.ReadKey();
        }

        private static Pet BuildDeepGraph()
        {
            return BuildDeepGraph(10);
        }

        private static Pet BuildDeepGraph(int recursion)
        {
            return new Pet
                       {
                           Father = (0 == recursion) ? null : BuildDeepGraph(recursion - 1)
                       };
        }

        private static Person LoadPerson(IObjectContainer db)
        {
            return (from Person p in db
                    select p).Single();
        }

        private static void AssertTrue(bool isTrue)
        {
            if (!isTrue)
            {
                Console.Out.WriteLine("ASSERT FAIL");
            }
        }

        private static IObjectContainer OpenDBWithTA(string path)
        {
            var cfg = Db4oEmbedded.NewConfiguration();
            cfg.Common.Add(new TransparentPersistenceSupport());
            var db = Db4oEmbedded.OpenFile(cfg, path);
            return db;
        }
    }

    [AttributeUsage(AttributeTargets.Class, AllowMultiple = false, Inherited = true)]
    internal class PersistedAttribute : Attribute
    {
    }

    internal enum Gender
    {
        Female,
        Male
    }

    [Persisted]
    internal class House
    {
        public string Name { get; set; }
    }

    [Persisted]
    internal class Address
    {
        public Address(string street, string city)
        {
            Street = street;
            City = city;
        }

        public string Street { get; private set; }

        public string City { get; private set; }
    }

    [Persisted]
    internal class Person
    {
        private IList<Address> _addresses = new ArrayList4<Address>();

        public IEnumerable<Address> Addresses
        {
            get { return _addresses; }
        }

        public void Add(Address item)
        {
            _addresses.Add(item);
        }

        public House House { get; set; }

        public Gender Gender { get; set; }

        public Pet MyPet { get; set; }
    }

    [Persisted]
    internal class Pet
    {
        public Pet Father { get; set; }
    }
}